# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - FormField Macro

    @copyright: 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin import wikiutil
from MoinSupport import *
from MoinForms import getFieldArguments, parseMacroArguments, getMacroArguments

Dependencies = ['pages']

escape = wikiutil.escape

# Macro functions.

def execute(macro, args):

    """
    Execute the 'macro' with the given 'args' to produce a form field element:

      * A field name
      * The WikiDict describing form fields

    The following optional named arguments are also supported:

      label=TEXT    The label employed by button-like fields
      path=PATH     The location of the field in the form section hierarchy
      fragment=NAME The name of the form region or fragment in the page

    The nature of each field is described by a WikiDict entry for the given
    field name.
    """

    request = macro.request
    fmt = macro.formatter
    page = fmt.page
    _ = request.getText

    # Interpret the arguments.

    parsed_args = parseMacroArguments(args)

    # Get special arguments.

    name, path, dictpage, label, section, fragment = getMacroArguments(parsed_args)

    if not name:
        return showError(_("No field name specified."), request)

    # Detect special modification fields.

    if name in ("_add", "_remove"):
        field_args = {"type" : "submit"}

        # The field name is a combination of the name, path and section.

        ref = "%s=%s%s%s" % (name, path or "", path and section and "/" or "", section or "")

    # Get the WikiDict and the field's definition.

    elif dictpage:
        wikidict = getWikiDict(dictpage, request)

        if not wikidict:
            return showError(_("WikiDict %s cannot be loaded for %s.") % (dictpage, name), request)

        try:
            field_definition = wikidict[name]
        except KeyError:
            return showError(_("No entry for %s in %s.") % (name, dictpage), request)

        field_args = getFieldArguments(field_definition)

        # The field name is a combination of the path and the name.

        ref = "%s%s" % (path and ("%s/" % path) or "", name)

    else:
        return showError(_("No WikiDict specified for %s.") % name, request)

    # Obtain any request parameters corresponding to the field.

    form = get_form(request)
    form_fragment = form.get("fragment", [None])[0]

    # Exclude values intended for other forms.

    if fragment and form_fragment != fragment or not fragment and form_fragment:
        values = []
        value = ""
    else:
        values = form.get(ref, [])
        value = form.get(ref, [""])[0]

    # Render the field.

    type = field_args.get("type", "text")

    if type == "text":
        return fmt.rawHTML('<input name="%s" type="%s" size="%s" value="%s" />' % (
            escattr(ref), escattr(type), escattr(field_args.get("size", "10")), escattr(value)
            ))

    elif type == "textarea":
        return fmt.rawHTML('<textarea name="%s" cols="%s" rows="%s">%s</textarea>' % (
            escattr(ref), escattr(field_args.get("cols", "60")), escattr(field_args.get("rows", "5")), escape(value)
            ))

    elif type == "submit":
        return fmt.rawHTML('<input name="%s" type="submit" value="%s" />' % (
            escattr(ref), escattr(_(label or name))
            ))

    elif type == "select":

        if not field_args.has_key("source"):
            return showError(_("No source dictionary given for %s.") % name, request)

        sourcedict = getWikiDict(field_args["source"], request)

        if not sourcedict:
            return showError(_("WikiDict %s cannot be loaded for %s.") % (sourcedict, name), request)

        maxselected = field_args.get("maxselected", [None])[0]
        maxselected = maxselected and int(maxselected) or maxselected

        output = []
        output.append(fmt.rawHTML('<select name="%s" %s>' % (escattr(ref), maxselected != 1 and "multiple" or "")))

        for option, label in sourcedict.items():
            is_selected = maxselected == 1 and option == value or maxselected != 1 and option in values

            output.append(fmt.rawHTML('<option value="%s" %s>%s</option>' % (
                escattr(option), is_selected and "selected" or "", escape(label))
                ))

        output.append(fmt.rawHTML('</select>'))
        return u''.join(output)

    else:
        return u''

def showError(text, request):
    fmt = request.formatter

    output = []
    append = output.append

    append(fmt.span(on=1, attrs={"class" : "form-field-error"}))
    append(fmt.text(text))
    append(fmt.span(on=0))

    return "".join(output)

# vim: tabstop=4 expandtab shiftwidth=4
